'use strict';
var callExpression = require('nodes');
var build = callExpression.build, nodes = callExpression.nodes;
var deshorthandify = require('./transform/shorthands').default;
var arrowify = require('./transform/arrow-functions').default;
var comprehendify = require('./transform/comprehensions').default;
var forofify = require('./transform/for-of').default;
var patternify = require('./transform/patterns').default;
var defaultify = require('./transform/default-params').default;
var classify = require('./transform/classes').default;
var restify = require('./transform/rest-param').default;
var spreadify = require('./transform/spread').default;
var templateify = require('./transform/template-literals').default;
var letify = require('./transform/let-declarations').default;
var modulize = require('./transform/modules').default;
var computify = require('./transform/computed-properties').default;
function transform(tree) {
  var program = build(tree);
  modulize(program);
  deshorthandify(program);
  arrowify(program);
  comprehendify(program);
  forofify(program);
  patternify(program);
  defaultify(program);
  classify(program);
  computify(program);
  restify(program);
  spreadify(program);
  templateify(program);
  letify(program);
  return program.toJSON();
}
exports.transform = transform;
exports.default = transform;
;